#include "anzeige.h"
#define UART_BAUD_RATE      1200

#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "libs/uart.h"
#include "libs/74HCT595.h"

//Daten für Schieberegister von 0 bis 9 (Index des Arrays)
#if HARDWARE_VERSION == 1
  unsigned char last[10] = {63, 6, 91, 79, 102, 109, 125, 7, 127, 111}; //Rechte Ziffer
  unsigned char first[10] = {63, 5, 91, 79, 101, 110, 126, 7, 127, 111}; //Linke Ziffer
#elif HARDWARE_VERSION == 2
  unsigned char last[10] = {63, 6, 91, 79, 102, 109, 125, 7, 127, 111}; //Rechte Ziffer
  unsigned char first[10] = {63, 6, 91, 79, 102, 109, 125, 7, 127, 111}; //Rechte Ziffer
#endif

volatile unsigned char count=0;
volatile unsigned int this_value[3] = {0,0,0}; //umwandeln und speichern des Wertes
volatile unsigned int last_value[3] = {0,0,0};
volatile unsigned char received_new = 0; //Variable zum Testen in der ISR

ISR (TIMER1_COMPA_vect) { //Wird mit 2Hz aufgerufen
  if ((PINB & (1<<5) && count>15) || count > 39) { //Nach 8 Sekunden oder 20 Sekunden
    if (!received_new) { //Anzeige nur löschen, wenn die Pistole keinen Wert mehr sendet. Ansonsten würde der Wert direkt wieder mit einem Flackern angezeigt
      HCT_load_byte(0); //Display ausschalten
      HCT_store();
      HCT_load_byte(0);
      HCT_store();
      count = 0;
      last_value[0] = 0;
      last_value[1] = 0;
      last_value[2] = 0;
    }
  } else ++count;

  received_new = 0; //Letzten Wert empfangen
}

void main (void) {
  DDRB |= (1<<3); //RS232 Pegelwandler einschalten
  PORTB |= (1<<3);

  DDRB &= ~(1<<5); //MOSI als Jumper für Wahl zwischen Anzeigedauer von 8s und 20s
  PORTB |=(1<<5);
  HCT_init();
  HCT_load_byte(first[8] + 128); //Selbstest (alle Segmente für 1 Sekunde ein)
  HCT_load_byte(last[8]);
  HCT_store();
  _delay_ms(1000);
  HCT_load_byte(0);
  HCT_load_byte(0);
  HCT_store();

  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU)); //UART -> RS232 initialisieren
  _delay_ms(10); //Zu schnelles Schreiben auf die Register senkt Zuverlässigkeit, da bereits zuvor durch die Fleury Lib geschrieben wird
  UBRRL = 0xA0;
  UBRRH = 0x01;
  _delay_ms(10);

  TCCR1A = (1<<WGM01); // CTC Modus -> Timer fuer automatisches Abschalten
  TCCR1B |= (1<<CS12)|(1<<CS10) |(1<<WGM12); // Prescaler 1024
  OCR1A = 3906;
  TIMSK |= (1<<OCIE1A); // -> Timer1 mit 2Hz
  sei();

  int c;
  unsigned char buffer[2][2], buffer_index; //Speichern der empfangenen Ziffern
  buffer[0][1] = '\0';
  buffer[1][1] = '\0';

  while (1) {
    c = uart_getc();
    if ( c & UART_NO_DATA ) {
    } else {
      if (c == 0x0d) { //Zeilenende
        if (buffer_index == 4) { //Alle erwarteten Zeichen empfangen?
          this_value[0] = atoi(buffer[0]); //int umwandlung
          this_value[1] = atoi(buffer[1]);
          this_value[2] = atoi(buffer[2]);

          if (this_value[0] || this_value[1] || this_value[2]) { //Nur reagieren, wenn Messwert != 000 ist
            received_new = 1; //Die Pistole sendet noch -> Abschalten der Anzeige sperren
            if (this_value[0] != last_value[0] || this_value[1] != last_value[1] || this_value[2] != last_value[2]) { //Nur reagieren, wenn sich der Wert geändert hat
              if (this_value[0] == 1) HCT_load_byte(first[this_value[1]] + 128); //Wenn 1 eingeschaltet werden soll
              else
                if (this_value[1] == 0) HCT_load_byte(0); //Wenn 1 nicht eingeschaltet werden soll und der Wert 8 oder 9 km/h ist
                else HCT_load_byte(first[this_value[1]]); //Sonst ohne +128 (Ausgang für 1)

              HCT_load_byte(last[this_value[2]]); //Rechte Ziffer
              HCT_store();
              count = 0; //Zähler zum Ausschalten zurücksetzen

              last_value[0] = this_value[0]; //Werte als last_value speichern
              last_value[1] = this_value[1];
              last_value[2] = this_value[2];
            }
          }
        }
        buffer_index = 0;
      } else {
        if (buffer_index < 3 && (c=='1' || c=='2' || c=='3' || c=='4' || c=='5' || c=='6' || c=='7' || c=='8' || c=='9' || c=='0')) { //Empfangen Wert auf Sinn prüfen
          buffer[buffer_index][0] = (unsigned char)c; //Ziffern zu Buffer hinzufügen
          ++buffer_index;
        } else if (buffer_index == 3 && c==':') //Doppelpunkt als Byte 4
          ++buffer_index;
        else //Falls unerwartetes Zeichen auftritt...
          buffer_index = 0; //...Empfang abbrechen und auf nächste Messung warten
      }
    }
  }
}
